package cz.muni.fi.pa165.rating;

import cz.muni.fi.pa165.common.BaseService;
import lombok.Getter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
public class RatingService extends BaseService<RatingDo> {

    @Getter
    private final RatingRepository repository;

    @Getter
    private final RatingMapper ratingMapper;

    /**
     * Constructs a new instance of the RatingService class.
     *
     * @param ratingRepository the repository for managing {@link RatingDo} entities
     * @param ratingMapper the mapper for converting between {@link RatingDo} and {@link RatingDto}
     */
    @Autowired
    public RatingService(RatingRepository ratingRepository, RatingMapper ratingMapper) {
        this.repository = ratingRepository;
        this.ratingMapper = ratingMapper;
    }

    /**
     * Retrieves all ratings.
     *
     * @return a list of all ratings
     */
    @Transactional(readOnly = true)
    public List<RatingDto> getRatings() {
        return repository.findAll().stream().map(ratingMapper::toDto).toList();
    }

    /**
     * Creates a new rating.
     *
     * @param ratingDto the rating DTO to be created
     */
    @Transactional
    public void createRating(RatingDto ratingDto) {
        RatingDo rating = ratingMapper.fromDto(ratingDto);
        repository.save(rating);
    }

    /**
     * Updates an existing rating.
     *
     * @param id        the ID of the rating to be updated
     * @param ratingDto the updated rating information
     */
    @Transactional
    public void updateRating(long id, RatingDto ratingDto) {
        ratingMapper.updateFromDto(ratingDto, repository.getReferenceById(id));
    }

    /**
     * Deletes an existing rating by ID.
     *
     * @param id the ID of the rating to be deleted
     */
    @Transactional
    public void deleteRating(long id) {
        repository.delete(repository.getReferenceById(id));
    }

    /**
     * Deletes all ratings.
     */
    @Transactional
    public void deleteAllRatings() {
        repository.deleteAll();
    }

    /**
     * Retrieves a rating by its ID.
     *
     * @param id the ID of the rating to retrieve.
     * @return the rating with the specified ID.
     */
    @Transactional(readOnly = true)
    public RatingDto getById(long id) {
        return ratingMapper.toDto(repository.getReferenceById(id));
    }
}
